@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2020
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -29,10 +29,10 @@ class HDEmbedder {
 protected:
    const Lattice<Decoration, SeqType>& HD;
    const Vector<double>& label_width;
-   int top_dim;
-   int bottom_dim;
-   int dim_delta;
-   typedef std::vector< std::vector<int> > layer_vector;
+   Int top_dim;
+   Int bottom_dim;
+   Int dim_delta;
+   typedef std::vector<std::vector<Int>> layer_vector;
    layer_vector layers;
    double total_width, epsilon;
    Vector<double> node_x, weights, nb_x_sum, width_in_layer;
@@ -42,21 +42,21 @@ public:
       : HD(HD_arg), label_width(label_width_arg),
         top_dim(HD.rank()), bottom_dim(HD.lowest_rank()), dim_delta(top_dim - bottom_dim), layers(dim_delta-1), node_x(HD.nodes()), weights(dim_delta), nb_x_sum(HD.nodes()), width_in_layer(dim_delta-1) {}
 
-   Matrix<double> compute(const perl::OptionSet& options);
+   Matrix<double> compute(const OptionSet& options);
 
 private:
-   void init(const perl::OptionSet& options);
+   void init(const OptionSet& options);
 
-   int try_move_node(std::vector<int>::iterator n,
-                     const std::vector<int>::iterator& n_first, const std::vector<int>::iterator& n_last,
+   Int try_move_node(std::vector<Int>::iterator n,
+                     const std::vector<Int>::iterator& n_first, const std::vector<Int>::iterator& n_last,
                      const double delta, const double *w);
 
-   inline void adjust_x(int node, const double x, const double *w);
-   inline bool good_swap(int lnode, int rnode, const double delta, const double *w) const;
+   inline void adjust_x(Int node, const double x, const double *w);
+   inline bool good_swap(Int lnode, Int rnode, const double delta, const double *w) const;
 };
 
 template <typename Decoration, typename SeqType>
-void HDEmbedder<Decoration, SeqType>::init(const perl::OptionSet& options)
+void HDEmbedder<Decoration, SeqType>::init(const OptionSet& options)
 {
    options["eps"] >> epsilon;
    const RandomSeed seed(options["seed"]);
@@ -67,13 +67,13 @@ void HDEmbedder<Decoration, SeqType>::init(const perl::OptionSet& options)
    {
       auto l = layers.begin();
       auto wd = width_in_layer.begin();
-      for (int d = bottom_dim + 1; d < top_dim; ++d, ++l, ++wd) {
-         const auto nodes=HD.nodes_of_rank(d);
-         const int n=nodes.size();
+      for (Int d = bottom_dim+1; d < top_dim; ++d, ++l, ++wd) {
+         const auto nodes = HD.nodes_of_rank(d);
+         const Int n = nodes.size();
          l->resize(n);
          copy_range(entire(random_permutation(nodes, random)), l->begin());
          accumulate_in(entire(select(label_width, nodes)), operations::max(), *wd);
-         assign_max(total_width, (*wd)*n);
+         assign_max(total_width, double(n)*(*wd));
       }
    }
 
@@ -84,10 +84,10 @@ void HDEmbedder<Decoration, SeqType>::init(const perl::OptionSet& options)
 
    auto wt = weights.begin();
    double prev_width = 1;
-   for (int d = bottom_dim + 1, l = 0;  d < top_dim;  ++d, ++wt,++l) {
-      const int layer_width = HD.nodes_of_rank(d).size();
+   for (Int d = bottom_dim+1, l = 0;  d < top_dim;  ++d, ++wt,++l) {
+      const double layer_width = double(HD.nodes_of_rank(d).size());
       const double width = total_width/layer_width;
-      double x = (width - total_width) / 2;
+      double x = (width - total_width)/2;
       for (auto n = entire(layers[l]); !n.at_end(); ++n, x += width)
          node_x[*n] = x;
       *wt=layer_width / prev_width;
@@ -96,9 +96,9 @@ void HDEmbedder<Decoration, SeqType>::init(const perl::OptionSet& options)
    *wt = 1 / prev_width;
 
    wt = weights.begin();
-   for (int d = bottom_dim + 1; d < top_dim; ++d, ++wt) {
+   for (Int d = bottom_dim+1; d < top_dim; ++d, ++wt) {
       for (auto nodes = entire(HD.nodes_of_rank(d)); !nodes.at_end(); ++nodes) {
-         const int n = *nodes;
+         const Int n = *nodes;
          nb_x_sum[n] = accumulate(select(node_x, HD.out_adjacent_nodes(n)), operations::add())*wt[1] +
             accumulate(select(node_x, HD.in_adjacent_nodes(n)), operations::add())/wt[0];
       }
@@ -106,11 +106,11 @@ void HDEmbedder<Decoration, SeqType>::init(const perl::OptionSet& options)
 }
 
 template <typename Decoration, typename SeqType>
-Matrix<double> HDEmbedder<Decoration, SeqType>::compute(const perl::OptionSet& options)
+Matrix<double> HDEmbedder<Decoration, SeqType>::compute(const OptionSet& options)
 {
    init(options);
 #if POLYMAKE_DEBUG
-   const bool debug_print = perl::get_debug_level() > 1;
+   const bool debug_print = get_debug_level() > 1;
 #endif
 
    // try to decrease the total tension
@@ -119,26 +119,26 @@ Matrix<double> HDEmbedder<Decoration, SeqType>::compute(const perl::OptionSet& o
       anyone_moved=false;
       Vector<double>::const_iterator wd=width_in_layer.begin(), wt=weights.begin();
       for (auto l=entire(layers); !l.at_end(); ++l, ++wd, ++wt) {
-         std::vector<int>::iterator n_first=l->begin(), n_last=l->end();
+         std::vector<Int>::iterator n_first = l->begin(), n_last = l->end();
 #if POLYMAKE_DEBUG
          if (debug_print) {
             cout << "layer " << (l-layers.begin()) << ":";
-            for (std::vector<int>::iterator n=n_first; n!=n_last; ++n) {
+            for (std::vector<Int>::iterator n = n_first; n != n_last; ++n) {
                cout << ' ' << *n << '(' << node_x[*n] << ')';
             }
             cout << endl;
          }
 #endif
-         for (std::vector<int>::iterator n=n_first; n!=n_last; ++n) {
+         for (std::vector<Int>::iterator n = n_first; n != n_last; ++n) {
 #if POLYMAKE_DEBUG
-            const int node=*n;
+            const Int node = *n;
 #endif
-            bool moved=try_move_node(n, n_first, n_last, *wd, &*wt);
+            const bool moved = try_move_node(n, n_first, n_last, *wd, &*wt);
             anyone_moved |= moved;
 #if POLYMAKE_DEBUG
             if (moved)
-               for (std::vector<int>::iterator n2=n_first+1; n2!=n_last; ++n2)
-                  if (node_x[*n2]-node_x[n2[-1]]+epsilon < *wd)
+               for (std::vector<Int>::iterator n2 = n_first+1; n2 != n_last; ++n2)
+                  if (node_x[*n2] - node_x[n2[-1]] + epsilon < *wd)
                      cerr << "ERROR after moving " << node << ": dist(" << n2[-1] << "," << *n2 << ") < " << *wd << endl;
 #endif
          }
@@ -148,7 +148,7 @@ Matrix<double> HDEmbedder<Decoration, SeqType>::compute(const perl::OptionSet& o
    Matrix<double> embedding(HD.nodes(), 2);
    const bool dual=options["dual"];
    double y=0;
-   for (int d=(dual ? top_dim-1 : bottom_dim+1), d_step=(dual ? -1 : 1), d_last=(dual ? bottom_dim: top_dim);
+   for (Int d = (dual ? top_dim-1 : bottom_dim+1), d_step = (dual ? -1 : 1), d_last = (dual ? bottom_dim: top_dim);
         d!=d_last;  d+=d_step, y+=1) {
       for (auto n=entire(HD.nodes_of_rank(d)); !n.at_end(); ++n) {
          embedding(*n,0)=node_x[*n];
@@ -161,22 +161,23 @@ Matrix<double> HDEmbedder<Decoration, SeqType>::compute(const perl::OptionSet& o
 }
 
 template <typename Decoration, typename SeqType>
-int HDEmbedder<Decoration, SeqType>::try_move_node(std::vector<int>::iterator n,
-                              const std::vector<int>::iterator& n_first, const std::vector<int>::iterator& n_last,
+Int HDEmbedder<Decoration, SeqType>::try_move_node(std::vector<Int>::iterator n,
+                              const std::vector<Int>::iterator& n_first, const std::vector<Int>::iterator& n_last,
                               const double delta, const double* wt)
 {
-   int node=*n;
-   int moved=0;
-   double x_min=nb_x_sum[node]/(HD.out_degree(node)*wt[1]+HD.in_degree(node)/wt[0]),
-      diff_to_min=node_x[node]-x_min;
-   if (diff_to_min>epsilon) {
+   Int node = *n;
+   Int moved = 0;
+   double x_min = nb_x_sum[node] / (double(HD.out_degree(node)) * wt[1] + double(HD.in_degree(node)) / wt[0]);
+   double diff_to_min = node_x[node] - x_min;
+
+   if (diff_to_min > epsilon) {
       // try moving to the left
       while (true) {
-         if (n--==n_first) {
+         if (n-- == n_first) {
             // the leftmost node, no neighbors to spring over
             assert(x_min >= -total_width/2-epsilon);
          } else {
-            const int node2=*n;
+            const Int node2 = *n;
             if (node_x[node2]+delta > x_min) {
                // gap between the neighbors is not large enough
                if (good_swap(node2, node, delta, wt)) {
@@ -208,7 +209,7 @@ int HDEmbedder<Decoration, SeqType>::try_move_node(std::vector<int>::iterator n,
             // the rightmost mode
             assert(x_min <= total_width/2+epsilon);
          } else {
-            const int node2=*n;
+            const Int node2 = *n;
             if (node_x[node2]-delta < x_min) {
                // gap between the neighbors is not large enough
                if (good_swap(node2, node, -delta, wt)) {
@@ -238,7 +239,7 @@ int HDEmbedder<Decoration, SeqType>::try_move_node(std::vector<int>::iterator n,
 }
 
 template <typename Decoration, typename SeqType>
-void HDEmbedder<Decoration, SeqType>::adjust_x(int node, const double x, const double* wt)
+void HDEmbedder<Decoration, SeqType>::adjust_x(Int node, const double x, const double* wt)
 {
    const double dx=x-node_x[node];
    node_x[node]=x;
@@ -249,15 +250,15 @@ void HDEmbedder<Decoration, SeqType>::adjust_x(int node, const double x, const d
 }
 
 template <typename Decoration, typename SeqType>
-bool HDEmbedder<Decoration, SeqType>::good_swap(int lnode, int rnode, const double delta, const double* wt) const
+bool HDEmbedder<Decoration, SeqType>::good_swap(Int lnode, Int rnode, const double delta, const double* wt) const
 {
-   return delta*( (delta+2*node_x[lnode])*( (HD.out_degree(lnode)-HD.out_degree(rnode))*wt[1] +
-                                            (HD.in_degree(lnode)-HD.in_degree(rnode))/wt[0] )
+   return delta*( (delta+2*node_x[lnode])*( double(HD.out_degree(lnode)-HD.out_degree(rnode))*wt[1] +
+                                            double(HD.in_degree(lnode)-HD.in_degree(rnode))/wt[0] )
                   - 2*(nb_x_sum[lnode]-nb_x_sum[rnode])) < 0;
 }
 
 template <typename Decoration, typename SeqType>
-Matrix<double> hd_embedder(perl::Object HD_obj, const Vector<double>& label_width, perl::OptionSet options)
+Matrix<double> hd_embedder(BigObject HD_obj, const Vector<double>& label_width, OptionSet options)
 {
    const Lattice<Decoration, SeqType> HD(HD_obj);
    HDEmbedder<Decoration, SeqType> HDE(HD, label_width);
