@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2020
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -36,13 +36,12 @@ using vector_list = Array<Vector<E>>;
 using graph_list = Array<Graph<Undirected>>;
 
 template<typename E>
-Matrix<E> list2matrix(const std::vector<Matrix<E> >& v, const int rows, const int cols)
+Matrix<E> list2matrix(const std::vector<Matrix<E>>& v, const Int r, const Int c)
 {
-    Matrix<E> A(rows,cols);
-    int i=0;
-    for (typename std::vector<Matrix<E> >::const_iterator it=v.begin(); it!=v.end(); ++it) {
-       for (int j=0; j<(*it).rows(); ++j,++i)
-          A.row(i)=(*it).row(j);
+    Matrix<E> A(r, c);
+    auto Arows = rows(A).begin();
+    for (const auto& M : v) {
+       Arows = copy_range(entire(rows(M)), Arows);
     }
     return A;
 }
@@ -51,7 +50,7 @@ template<typename E>
 E solve_lp_mixed_volume(const Matrix<E>& M, const Vector<E>& objective)
 {
     Matrix<E> eqs(M.cols()-1, M.cols());
-    for (int j=0; j<M.cols()-1; ++j)
+    for (Int j = 0; j < M.cols()-1; ++j)
        eqs.row(j) = unit_vector<E>(M.cols(),j+1);
     const auto S = solve_LP(eqs, M, objective, true);
     if (S.status != LP_status::valid)
@@ -60,76 +59,76 @@ E solve_lp_mixed_volume(const Matrix<E>& M, const Vector<E>& objective)
 }
 
 template<typename E>
-Matrix<E> construct_A(const int n, const Array<int>& r, const matrix_list<E>& polytopes, const vector_list<E>& lifted_edges)
+Matrix<E> construct_A(const Int n, const Array<Int>& r, const matrix_list<E>& polytopes, const vector_list<E>& lifted_edges)
 {
     std::vector<Matrix<E> > c;
-    int R=0;
-    for (int i=0; i<n; ++i) {
+    Int R = 0;
+    for (Int i = 0; i < n; ++i) {
        Matrix<E> B = zero_matrix<E>(lifted_edges[i].size(),n+1);
-       B.col(0)=lifted_edges[i];
-       B.col(i+1)=ones_vector<E>(r[i]);
+       B.col(0) = lifted_edges[i];
+       B.col(i+1) = ones_vector<E>(r[i]);
        c.push_back((polytopes[i]| B));
        R+=r[i];
     }
-    Matrix<E> A=list2matrix(c,R,n+2+n);
-    Vector<E> vec =unit_vector<E>(A.cols(),polytopes[0].cols());
-    A = (A / vec);
+    Matrix<E> A = list2matrix(c, R, 2*n+2);
+    Vector<E> vec = unit_vector<E>(A.cols(),polytopes[0].cols());
+    A /= vec;
     return T(A.minor(All,sequence(1,A.cols()-1))); // slice ones and transpose
 }
 
 template<typename E>
-bool lower_envelope_check(Matrix<E>& A, const int n, const int k, const Array<int>& r, const Vector<E>& m)
+bool lower_envelope_check(Matrix<E>& A, const Int n, const Int k, const Array<Int>& r, const Vector<E>& m)
 {
-    int R=0;
-    assert(r.size()>=k);
-    for (int i=0; i<k; ++i)
-       R+=r[i];
-    const Vector<E> b = ( m | ones_vector<E>(n));
+    Int R = 0;
+    assert(r.size() >= k);
+    for (Int i = 0; i < k; ++i)
+       R += r[i];
+    const Vector<E> b = m | ones_vector<E>(n);
     const Vector<E> obj = unit_vector<E>(R+2,R+1);
-    T(A).row(0)=-b;
+    T(A).row(0) = -b;
 
     const Matrix<E> M(A.minor(sequence(0,n+k+1),~sequence(R+1,A.cols()-R-2)));
     return solve_lp_mixed_volume(M, obj)==0;
 }
 
 template<typename E>
-E volume(const int n, const Array<int>& node, const Array<int>& next, const matrix_list<E>& polytopes, const graph_list& graphs)
+E volume(const Int n, const Array<Int>& node, const Array<Int>& next, const matrix_list<E>& polytopes, const graph_list& graphs)
 {
-    Matrix<E> A(1,polytopes[0].cols());
-    for (int j=0; j<n; ++j) {
-       auto it=entire(graphs[j].adjacent_nodes(node[j]));
-       for (int i=0; i<next[j]; ++i)  //reset iterator
+    Matrix<E> A(n, polytopes[0].cols()-1);
+    for (Int j = 0; j < n; ++j) {
+       auto it = entire(graphs[j].adjacent_nodes(node[j]));
+       for (Int i = 0; i < next[j]; ++i)  //reset iterator
           ++it;
-       A = ( A / (polytopes[j].row(node[j])-polytopes[j].row(*it)) );
+       A.row(j) = (polytopes[j].row(node[j]) - polytopes[j].row(*it)).slice(range_from(1));
     }
-    E d = det(A.minor(sequence(1,A.rows()-1),sequence(1,A.cols()-1)));
-    if (d==0)
+    E d = det(A);
+    if (d == 0)
        throw std::runtime_error("mixed_volume: calculation failed, edge matrix is singular.");
     // check (2.9) in [2]. change the  Lift-functions
     return abs(d);
 }
 
 template <typename E>
-E mixed_volume(const Array<perl::Object>& summands)
+E mixed_volume(const Array<BigObject>& summands)
 {
    E vol(0);             // mixedVolume
-   const int n = summands.size();      // number of (input)polytopes
+   const Int n = summands.size();      // number of (input)polytopes
    matrix_list<E> polytopes(n);      // stores matrices s.t. the i-th entry is a discribtion of P_j by vertices
    vector_list<E> lifted_edges(n);
    graph_list graphs(n);         // stores all graphs from the input polytopes P_j
-   Array<int> node(n);
-   Array<int> next(n);
-   Array<int> r(n);              //stores the number of input Points
+   Array<Int> node(n);
+   Array<Int> next(n);
+   Array<Int> r(n);              //stores the number of input Points
 
    //initialization:
-   int j=0;
+   Int j = 0;
    Vector<E> Lift(n+1);
-   Lift[0]=0;
-   for (const perl::Object& s : summands) {
-      const Matrix<E> m=s.give("VERTICES");
-      polytopes[j]=m;
-      r[j]=m.rows();
-      for (int k=1; k<=n; ++k)        //LIFT
+   Lift[0] = 0;
+   for (const BigObject& s : summands) {
+      const Matrix<E> m = s.give("VERTICES");
+      polytopes[j] = m;
+      r[j] = m.rows();
+      for (Int k = 1; k <= n; ++k)        //LIFT
          Lift[k] = 1 + j*(1 - j*(1 - k*j)); //1 + j - j*j + k*j*j*j;
       lifted_edges[j] = m*Lift;
       const Graph<Undirected> graph=s.give("GRAPH.ADJACENCY");
@@ -142,43 +141,41 @@ E mixed_volume(const Array<perl::Object>& summands)
    A = ones_vector<E>(A.rows()) | A;
 
    // Algo:
-   Vector<E> m=zero_vector<E>(n+1);
+   Vector<E> m = zero_vector<E>(n+1);
    Vector<E> temp;
-   int count;
+   Int count;
 
-   j=0;
-   for (int i=0; i<polytopes[j].rows(); ++i) {
-      auto it=entire(graphs[j].adjacent_nodes(i));
-      for (count=0; count<polytopes[j].rows()-1; ++count) {
-         if (*it>i) {
-            temp = (polytopes[j].row(i)+polytopes[j].row(*it))/2;
-            temp = (temp | (((lifted_edges[j])[i]+(lifted_edges[j])[*it])/2));
-            temp = temp.slice(range_from(1));
+   j = 0;
+   for (Int i = 0; i < polytopes[j].rows(); ++i) {
+      auto it = entire(graphs[j].adjacent_nodes(i));
+      for (count = 0; count < polytopes[j].rows()-1; ++count) {
+         if (*it > i) {
+            temp = ((polytopes[j].row(i) + polytopes[j].row(*it))/2).slice(range_from(1))
+                 | (lifted_edges[j][i] + lifted_edges[j][*it])/2;
             if (lower_envelope_check(A, n, j+1, r, Vector<E>(m+temp))) {
-               node[j]=i;
-               next[j]=count;
-               if (j==n-1) {
-                  vol+=volume(n,node,next,polytopes,graphs);
+               node[j] = i;
+               next[j] = count;
+               if (j == n-1) {
+                  vol += volume(n, node, next, polytopes, graphs);
                } else {
                   ++j;
-                  m+=temp;
-                  count=polytopes[j].rows();  //jump out of loop
-                  i=-1;
+                  m += temp;
+                  count = polytopes[j].rows();  //jump out of loop
+                  i = -1;
                }
             }
          }
          ++it;
-         if (j>0 and it.at_end() and i==polytopes[j].rows()-1) {
+         if (j > 0 && it.at_end()&& i == polytopes[j].rows()-1) {
             --j;
             i = node[j];
             it = entire(graphs[j].adjacent_nodes(i));
             count = next[j];
-            for (int k=0; k<count; ++k) //reset iterator
+            for (Int k = 0; k < count; ++k) //reset iterator
                ++it;
-            temp = (polytopes[j].row(i)+polytopes[j].row(*it))/2;
-            temp = (temp | (((lifted_edges[j])[i]+(lifted_edges[j])[*it])/2));
-            temp = temp.slice(range_from(1));
-            m = m-temp;
+            temp = ((polytopes[j].row(i) + polytopes[j].row(*it))/2).slice(range_from(1))
+                 | (lifted_edges[j][i] + lifted_edges[j][*it])/2;
+            m -= temp;
             ++it;
          }
          if (it.at_end())
